@@ -788,10 +788,15 @@ def get_axes(ax=None, which='cx'):
 
     :return: Axes instance or 2D array of Axes instances
     """
-    if ax is None:
-        out_loc, settings_loc, out, settings, parent_settings, scratch = pick_output_location()
+    out_loc, settings_loc, out, settings, parent_settings, scratch = pick_output_location()
+    if which == 'cx' and not settings.get('show_cx', True):
+        printd(f'show_cx is off, so no cx_ax is needed. Returning None...', topic='efitviewer')
+        return None
+    if which == 'profile' and settings.get('profiles', {}).get('num_profiles', 0) < 1:
+        printd(f'num_profiles < 1, so no profile_axes are needed. Returning None...', topic='efitviewer')
 
-        printd('plot_single_cx_ods(): trying to look up efitviewer GUI figure...', topic='efitviewer')
+    if ax is None:
+        printd(f'plot_single_cx_ods(): trying to look up efitviewer axes for {which}...', topic='efitviewer')
         if which == 'profile':
             ax = scratch.get('efitviewer_profile_axs', None)
             if ax is not None and len(ax.flatten()):
@@ -802,6 +807,7 @@ def get_axes(ax=None, which='cx'):
             ax = scratch.get('efitviewer_cx_ax', None)
             test_ax = ax
     else:
+        printd(f'received some axes to check against the needs of {which}...', topic='efitviewer')
         test_ax = ax
         if which == 'profile':
             if len(test_ax.flatten()):
@@ -809,6 +815,7 @@ def get_axes(ax=None, which='cx'):
             else:
                 test_ax = None
     if test_ax is not None:
+        printd(f'testing whether {which} axes have a figure associated with them', topic='efitviewer')
         # Check if ax is active
         try:
             # https://stackoverflow.com/a/15311949/6605826
@@ -823,13 +830,24 @@ def get_axes(ax=None, which='cx'):
             printd('plot_single_cx_ods(): efitviewer figure seems to have been closed', topic='efitviewer')
             ax = None
     if ax is None:
-        printd('plot_single_cx_ods(): using gca() to find axes', topic='efitviewer')
-        ax = pyplot.gca()
+        printd('plot_single_cx_ods(): using populate_efitviewer_figure()', topic='efitviewer')
+        fig = scratch['efitviewer_fig'] = pyplot.figure()
+        settings.setdefault('profiles', SettingsName())
+        cx_ax, profile_axs = populate_efitviewer_figure(
+            fig,
+            scratch,
+            show_cx=settings.setdefault('show_cx', True),
+            num_profiles=settings['profiles'].setdefault('num_profiles', 0),
+            num_profile_cols=settings['profiles'].setdefault('num_profile_cols', 0),
+            profiles_sharex=settings['profiles'].setdefault('sharex', 'all'),
+            profiles_sharey=settings['profiles'].setdefault('sharey', 'none'),
+            **settings['profiles'].setdefault('gridspec_kw', {}),
+        )
         if which == 'profile':
-            # Not sure if this is the best solution, but it should work for now
-            ax_ = np.empty((1, 1), object)
-            ax_[0, 0] = ax
-            ax = ax_
+            ax = profile_axs
+        else:
+            ax = cx_ax
+
     return ax
 
 
@@ -872,6 +890,8 @@ def plot_style_enforcement(ax=None, plot_style=None):
     if plot_style is None:
         plot_style = settings.get('plot_style_kw', {})
     ax = get_axes(ax)
+    if ax is None:
+        return
     fig = ax.figure
 
     # Frame
@@ -989,7 +1009,10 @@ def populate_efitviewer_figure(
         cx_ax = scratch['efitviewer_cx_ax'] = None
 
     profile_axs = scratch['efitviewer_profile_axs'] = np.empty((num_rows, num_profile_cols_), dtype=object)
-    profile_axs[:] = None
+    for i in range(num_rows):
+        for j in range(num_profile_cols_):
+            profile_axs[i, j] = None
+
     for i in range(num_profiles):
         row = i % num_rows
         col = i // num_rows
@@ -1013,7 +1036,10 @@ def populate_efitviewer_figure(
         else:
             sharey = None
         profile_axs[row, col] = fig.add_subplot(gs[row, fig_col], sharex=sharex, sharey=sharey)
-
+        if (row < (num_rows - 1)) and (profiles_sharex in ['col', 'all']):
+            profile_axs[row, col].get_xaxis().set_visible(False)
+        if (col > 0) and profiles_sharey in ['row', 'all']:
+            profile_axs[row, col].get_yaxis().set_visible(False)
     return cx_ax, profile_axs
 
 
@@ -1021,6 +1047,9 @@ def plot_cx_ods(ods_idx=None, odss=None, cases=None, **kw):
     r"""
     Wrapper for making multiple calls to plot_single_cx_ods()
 
+    Note that plot_single_cx_ods() calls plot_single_time_profile_set(),
+    so this is the umbrella for all CX and profile plots.
+
     :param ods_idx: int or list of ints [optional]
         You should probably let it do its default behavior, but you can override it if you need to.
 
@@ -1041,6 +1070,8 @@ def plot_cx_ods(ods_idx=None, odss=None, cases=None, **kw):
     """
     out_loc, settings_loc, out, settings, parent_settings, scratch = pick_output_location()
 
+    printd('started plot_cx_ods...', topic='efitviewer')
+
     if cases is None:
         cases = settings['cases']
     if ods_idx is None:
@@ -1098,10 +1129,235 @@ def plot_cx_ods(ods_idx=None, odss=None, cases=None, **kw):
     return
 
 
-def plot_single_time_profile_set(ods=None, t_idx=0, profile_axs=None, profile_x_quantity=None, profile_y_quantities=None, gentle=False):
+def calc_btor_2d(ods, t_idx):
+    """
+    Calculates the toroidal magnetic field on the 2D grid
+
+    :param ods: ODS instance
+
+    :param t_idx: int
+    """
+    s = ods['equilibrium.time_slice'][t_idx]
+    t = ods['equilibrium.time']
+    bcentr = interp1d(t, ods['equilibrium.vacuum_toroidal_field.b0'])(s['time'])
+    rcentr = ods['equilibrium.vacuum_toroidal_field.r0']
+    rgrid = s['profiles_2d.0.grid.dim1']
+    zgrid = s['profiles_2d.0.grid.dim2']
+    btvac = bcentr * rcentr / rgrid
+    btvac2d = btvac[:, np.newaxis] + zgrid[np.newaxis, :] * 0
+
+    psi1d = s['profiles_1d.psi']
+    f1d = s['profiles_1d.f']
+    psi2d = s['profiles_2d.0.psi']
+    f2d = interp1d(psi1d, f1d, bounds_error=False, fill_value=np.NaN)(psi2d)
+    btor = f2d / rgrid[:, np.newaxis]
+    btor[np.isnan(btor)] = btvac2d[np.isnan(btor)]
+    s['profiles_2d.0.b_field_tor'] = btor
+
+
+def calculate_j_toroidal(ods, t_idx, check_availability=False):
+    """
+    Calculates toroidal current density
+
+    :param ods: ODS instance
+
+    :param t_idx: int
+
+    :param check_availability: bool
+        Return a bool flag indicating whether the calculation is possible instead of doing the calculation
+
+    :return: 1D float array
+    """
+    from omfit_classes.fluxSurface import fluxSurfaces
+
+    scratch = pick_output_location()[5]
+
+    profiles_1d = ods['equilibrium']['time_slice'][t_idx]['profiles_1d']
+    profiles_2d = ods['equilibrium']['time_slice'][t_idx]['profiles_2d'][0]
+    global_quantities = ods['equilibrium.time_slice'][t_idx]['global_quantities']
+    required_2d = ['psi']
+    required_1d = ['psi', 'f']
+    required_global = ['magnetic_axis.r']
+
+    is_available = all(
+        [r in profiles_1d for r in required_1d]
+        + [r in profiles_2d for r in required_2d]
+        + [r in global_quantities for r in required_global]
+    )
+    if check_availability:
+        return is_available
+
+    if (profiles_1d.get('r_outboard', None) is None) or (not len(profiles_1d['r_outboard'])) or np.all(np.isnan(profiles_1d['r_outboard'])):
+        define_r_outboard(ods, t_idx)
+    if (
+        (profiles_2d.get('b_field_tor', None) is None)
+        or (not len(profiles_2d['b_field_tor']))
+        or np.all(np.isnan(profiles_2d['b_field_tor']))
+    ):
+        calc_btor_2d(ods, t_idx)
+
+    fluxtag = f"flux_surfaces_{ods['dataset_description.data_entry.machine']}#{ods['dataset_description.data_entry.pulse']}_{t_idx}"
+    fluxsurf = scratch[fluxtag] = fluxSurfaces(
+        Rin=profiles_2d['grid.dim1'],
+        Zin=profiles_2d['grid.dim2'],
+        PSIin=profiles_2d['psi'].T,
+        Btin=profiles_2d['b_field_tor'].T,
+        Rcenter=global_quantities['magnetic_axis.r'],
+        F=profiles_1d['f'],
+        P=profiles_1d['pressure'],
+        rlim=ods['wall.description_2d.0.limiter.unit.0.outline.r'],
+        zlim=ods['wall.description_2d.0.limiter.unit.0.outline.z'],
+        levels=np.linspace(0, 1, len(profiles_1d['psi'])),
+        gEQDSK=None,
+        cocosin=ods.cocos,
+    )
+    profiles_1d['j_tor'] = fluxsurf['avg']['Jt']
+
+
+def define_r_outboard(ods, t_idx=None):
+    """
+    Given an ODS with an equilibrium, define the 1D profile quantity r_outboard is defined.
+
+    :param ods: ODS instance
+
+    :param t_idx: int
+        Index of the time slice to work on
+    """
+    profiles_1d = ods['equilibrium']['time_slice'][t_idx]['profiles_1d']
+    profiles_2d = ods['equilibrium']['time_slice'][t_idx]['profiles_2d'][0]
+    global_quantities = ods['equilibrium.time_slice'][t_idx]['global_quantities']
+    r = profiles_2d['grid.dim1']
+    z = profiles_2d['grid.dim2']
+    psi2d = profiles_2d['psi']
+    rmaxis = global_quantities['magnetic_axis.r']
+    zmaxis = global_quantities['magnetic_axis.z']
+    zidx = abs(z - zmaxis).argmin()
+    rsel = r >= rmaxis
+    psi_omp = psi2d[rsel, zidx]
+    r_slice = r[rsel]
+    profiles_1d['r_outboard'] = interp1d(psi_omp, r_slice, bounds_error=False, fill_value='extrapolate')(profiles_1d['psi'])
+
+
+def take_slice_of_2d(quantity, ods, t_idx):
+    """
+    Takes a 1D slice out of a 2D profile quantity
+
+    :param quantity: str
+
+    :param ods: ODS instance
+
+    :param t_idx: int
+
+    :return: 1D float array
+    """
+    profiles_1d = ods['equilibrium']['time_slice'][t_idx]['profiles_1d']
+    profiles_2d = ods['equilibrium']['time_slice'][t_idx]['profiles_2d'][0]
+    global_quantities = ods['equilibrium.time_slice'][t_idx]['global_quantities']
+    if (profiles_1d.get('r_outboard', None) is None) or (not len(profiles_1d['r_outboard'])) or np.all(np.isnan(profiles_1d['r_outboard'])):
+        define_r_outboard(ods, t_idx)
+    rtar = profiles_1d['r_outboard']
+    ztar = global_quantities['magnetic_axis.z'] + 0 * rtar
+    return RectBivariateSpline(profiles_2d['grid.dim1'], profiles_2d['grid.dim2'], profiles_2d[quantity])(rtar, ztar, grid=False)
+
+
+def get_quantity(quantity, ods, t_idx):
+    """
+    Intercept quantity request to calculate special quantities as needed
+
+    :param quantity: str
+
+    :param ods: ODS instance
+
+    :param t_idx: int
+
+    :return: 1D float array
+    """
+    profiles_1d = ods['equilibrium']['time_slice'][t_idx]['profiles_1d']
+    profiles_2d = ods['equilibrium']['time_slice'][t_idx]['profiles_2d'][0]
+    global_quantities = ods['equilibrium.time_slice'][t_idx]['global_quantities']
+    if quantity == 'b_field_tor' and quantity not in profiles_2d:
+        calc_btor_2d(ods, t_idx)
+    if quantity == 'j_tor' and quantity not in profiles_1d and calculate_j_toroidal(ods, t_idx, check_availability=True):
+        calculate_j_toroidal(ods, t_idx)
+    if quantity == 'psi_norm':
+        q = (profiles_1d['psi'] - global_quantities['psi_axis']) / (global_quantities['psi_boundary'] - global_quantities['psi_axis'])
+    elif (quantity not in profiles_1d) and (quantity in profiles_2d):
+        q = take_slice_of_2d(quantity, ods, t_idx)
+    else:
+        q = profiles_1d[quantity]
+    return q
+
+
+def constraint_profile_mse_polarisation_angle(ods, t_idx=0, profile_x_quantity='R', ax=None, **plot_kw):
+    """
+    Plots a profile of the MSE polarization angle constraint
+
+    Each constraint profile plot function must accept these keywords: [ods, t_idx, profile_x_quantity, ax]
+    Each constraint profile plot function must be named exactly:
+        'constraint_profile_{name of constraint set in equilibrium}'
+
+    :param ods: ODS instance
+
+    :param t_idx: int
+
+    :param profile_x_quantity: str
+
+    :param ax: Axes instance
+
+    :param plot_kw: Addtional keywords passed to plot and errorbar
+    """
+    if ax is None:
+        # Only relevant for testing; otherwise a specific set of Axes within the
+        # profile axes set should always be provided by the calling function.
+        ax = gca()
+    mse_eq = ods['equilibrium.time_slice'][t_idx]['constraints.mse_polarisation_angle']
+    nchan = len(mse_eq)
+    r = np.empty(nchan)
+    z = np.empty(nchan)
+    measured = np.empty(nchan)
+    measured_err = np.empty(nchan)
+    reconstructed = np.empty(nchan)
+    for ch in range(nchan):
+        r[ch] = ods[mse_eq[ch]['source']]['active_spatial_resolution.0.centre.r']
+        z[ch] = ods[mse_eq[ch]['source']]['active_spatial_resolution.0.centre.z']
+        measured[ch] = mse_eq[ch]['measured']
+        measured_err[ch] = mse_eq[ch]['measured_error_upper']
+        reconstructed[ch] = mse_eq[ch]['reconstructed']
+
+    if profile_x_quantity.lower() == 'r':
+        x = r
+        ax.set_xlabel('R / m')
+    elif profile_x_quantity.lower() == 'z':
+        x = z
+        ax.set_xlabel('Z / m')
+    else:
+        profiles_1d = ods['equilibrium']['time_slice'][t_idx]['profiles_1d']
+        profiles_2d = ods['equilibrium']['time_slice'][t_idx]['profiles_2d'][0]
+        psi = RectBivariateSpline(profiles_2d['grid.dim1'], profiles_2d['grid.dim2'], profiles_2d['psi'])(r, z, grid=False)
+        x = interp1d(profiles_1d['psi'], get_quantity(profile_x_quantity, ods, t_idx), bounds_error=False, fill_value=np.NaN)(psi)
+        ax.set_xlabel(omas_eq1d_pretty_name(profile_x_quantity))
+
+    ax.set_ylabel('MSE pitch angle / radians')
+
+    pkw = copy.deepcopy(plot_kw)
+    for naughty in ['linestyle', 'marker', 'ls', 'label']:
+        pkw.pop(naughty, None)
+
+    a = ax.errorbar(x, measured, measured_err, marker='x', linestyle='', label='Measured', **pkw)
+    # Don't use the next color in the cycle or else these profile colors won't line up with equilibrium colors
+    pkw['color'] = associated_color(a[0].get_color())
+    ax.plot(x, reconstructed, marker='.', linestyle='', label='Reconstructed', **pkw)
+    ax.legend(loc=0)
+
+
+def plot_single_time_profile_set(
+    ods=None, t_idx=0, profile_axs=None, profile_x_quantity=None, profile_y_quantities=None, gentle=False, **plotkw
+):
     """
     Plots profiles for a single time-slice
 
+    This is normally called by plot_single_cx_ods()
+
     :param ods: ODS() instance with the data
 
     :param t_idx: int
@@ -1114,28 +1370,51 @@ def plot_single_time_profile_set(ods=None, t_idx=0, profile_axs=None, profile_x_
 
     :param gentle: bool
         Report exceptions with print and ax.text instead of raising
+
+    :param plotkw: Additional keywords passed to plot and plot-related functions in constraint profiles
+        Could be passed to errorbar(), for example.
     """
     if profile_axs is None or profile_x_quantity is None:
         return
+    settings = pick_output_location()[3]
+
     profiles_1d = ods['equilibrium']['time_slice'][t_idx]['profiles_1d']
+    profiles_2d = ods['equilibrium']['time_slice'][t_idx]['profiles_2d'][0]
+    constraints = ods['equilibrium']['time_slice'][t_idx]['constraints']
     global_quantities = ods['equilibrium.time_slice'][t_idx]['global_quantities']
 
-    def get_quantity(quantity):
-        """Intercept quantity request to calculate special quantities as needed"""
-        if quantity == 'psi_norm':
-            q = (profiles_1d['psi'] - global_quantities['psi_axis']) / (global_quantities['psi_boundary'] - global_quantities['psi_axis'])
-        else:
-            q = profiles_1d[quantity]
-        return q
+    if (profile_x_quantity == 'r_outboard') and ('r_outboard' not in profiles_1d):
+        define_r_outboard(ods, t_idx)
+
+    # Sanitize plotkw
+    pkw = copy.deepcopy(plotkw)
+    for naughty in ['linewidths', 'colors', 'linestyles']:
+        pkw.pop(naughty, None)
 
     for i, ax in enumerate(profile_axs.flatten()):
         if profile_y_quantities[i] is None:
             return
-        ax.set_xlabel(omas_eq1d_pretty_name(profile_x_quantity))
-        ax.set_ylabel(omas_eq1d_pretty_name(profile_y_quantities[i]))
+
+        # Is it a constraint that needs a special function?
+        if (
+            (profile_y_quantities[i] not in profiles_1d)
+            and (profile_y_quantities[i] not in profiles_2d)
+            and (profile_y_quantities[i] in constraints)
+        ):
+            constraint_profile_function_name = f'constraint_profile_{profile_y_quantities[i]}'
+        else:
+            constraint_profile_function_name = None
+
+        if constraint_profile_function_name is None:
+            ax.set_xlabel(omas_eq1d_pretty_name(profile_x_quantity))
+            ax.set_ylabel(omas_eq1d_pretty_name(profile_y_quantities[i]))
         try:
-            ax.plot(get_quantity(profile_x_quantity), get_quantity(profile_y_quantities[i]))
-        except (KeyError, ValueError):
+            if constraint_profile_function_name is not None:
+                constraint_profile_function = eval(constraint_profile_function_name)
+                constraint_profile_function(ods=ods, t_idx=t_idx, profile_x_quantity=profile_x_quantity, ax=ax, **pkw)
+            else:
+                ax.plot(get_quantity(profile_x_quantity, ods, t_idx), get_quantity(profile_y_quantities[i], ods, t_idx), **pkw)
+        except (KeyError, ValueError, NameError):
             if gentle:
                 # Display the error instead of bringing the whole GUI down if the plot fails.
                 print('Error in plot_single_time_profile_set:')
@@ -1149,6 +1428,21 @@ def plot_single_time_profile_set(ods=None, t_idx=0, profile_axs=None, profile_x_
             else:
                 raise
 
+    if settings['profiles'].get('sharex', 'all') in ['all', 'col']:
+        if len(profile_axs[:, 0]) > 1:
+            for ax in profile_axs[:-1, :].flatten():
+                ax.set_xlabel('')
+    if settings['profiles'].get('sharey', 'none') in ['all', 'row']:
+        if len(profile_axs[0, :]) > 1:
+            for ax in profile_axs[:, 1:].flatten():
+                ax.set_ylabel('')
+
+    if len(np.shape(profile_axs)) > 1 and np.shape(profile_axs)[1] > 1 and settings['profiles'].setdefault('last_col_right_label', True):
+        for ax in profile_axs[:, -1].flatten():
+            ax.yaxis.tick_right()
+            ax.yaxis.set_ticks_position('both')
+            ax.yaxis.set_label_position("right")
+
 
 def plot_single_cx_ods(
     cx_ax=None,
@@ -1182,10 +1476,12 @@ def plot_single_cx_ods(
     """
     Plots efitviewer-style cross section with overlays
 
-    If data for an overlay are missing, they will be added before plotting
+    If data for an overlay are missing, they will be added before plotting.
+
+    This function also calls plot_single_time_profile_set() to add profiles to the figure.
 
     :param cx_ax: Axes instance
-        Axes to draw cross section on. Defaults to gca() if not specified.
+        Axes to draw cross section on.
 
     :param profile_axs: 2D array containing Axes instances
         Axes to use for drawing profiles.
@@ -1271,8 +1567,8 @@ def plot_single_cx_ods(
         Settings for enforcing plot axes style
 
     :param eqkw: dict-like
-        Customization options for equilibrium overlay contours,
-        passed to plot for boundary & contour for flux surfaces
+        Customization options for equilibrium overlay contours and profile plots,
+        passed to plot for boundary & contour for flux surfaces and to profiles.
 
     :param xkw: dict-like
         Customization options for equilibrium overlay's x-point, such as marker, color, markersize, mew, ...
@@ -1299,15 +1595,40 @@ def plot_single_cx_ods(
     if timing_ref is not None:
         printe('-' * 80)
 
+    if clear_first and cx_ax is None and profile_axs is None:
+        fig = scratch.setdefault('efitviewer_fig', None)
+        if fig is None:
+            fig = scratch['efitviewer_fig'] = pyplot.gcf()
+        for ax in fig.axes:
+            ax.remove()
+        populate_efitviewer_figure(
+            fig,
+            scratch,
+            show_cx=settings.setdefault('show_cx', True),
+            num_profiles=settings['profiles'].setdefault('num_profiles', 0),
+            num_profile_cols=settings['profiles'].setdefault('num_profile_cols', 0),
+            profiles_sharex=settings['profiles'].setdefault('sharex', 'all'),
+            profiles_sharey=settings['profiles'].setdefault('sharey', 'none'),
+            **settings['profiles'].setdefault('gridspec_kw', {}),
+        )
+
     cx_ax = get_axes(cx_ax)
-    fig = cx_ax.figure
+    if isinstance(cx_ax, matplotlib.axes.Axes):
+        fig = cx_ax.figure
+    else:
+        fig = None
     profile_axs = get_axes(profile_axs, which='profile')
+    if fig is None and profile_axs is not None and isinstance(profile_axs[0, 0], matplotlib.axes.Axes):
+        fig = profile_axs[0, 0].figure
+    elif fig is None:
+        fig = scratch.get('efitviewer_fig', None)
     if clear_first:
         if cx_ax is not None:
             cx_ax.cla()
         if profile_axs is not None:
             for ax in profile_axs.flatten():
-                ax.cla()
+                if ax is not None:
+                    ax.cla()
 
     # noinspection PyBroadException
     try:
@@ -1322,7 +1643,7 @@ def plot_single_cx_ods(
 
         # Resolve keyword default values
         if contour_quantity is None:
-            contour_quantity = settings.setdefault('contour_quantity', 'psi')
+            contour_quantity = settings.setdefault('contour_quantity', 'psi_norm')
         if allow_fallback is None:
             allow_fallback = settings.setdefault('allow_fallback', False)
         if levels is None:
@@ -1429,17 +1750,19 @@ def plot_single_cx_ods(
                     if compare_version(ver, omas.__version__) > 0:
                         # Version required to support this keyword is newer than the OMAS version in use, so skip.
                         eqcxkw.pop(kw, None)
-
-                ods.plot_equilibrium_CX(**eqcxkw)  # The call to draw the equilibrium contours
-                plot_single_time_profile_set(
-                    ods=ods,
-                    t_idx=it,
-                    profile_axs=profile_axs,
-                    profile_x_quantity=profile_x_quantity,
-                    profile_y_quantities=profile_y_quantities,
-                    gentle=gentle,
-                )
-        else:
+                if cx_ax is not None:
+                    ods.plot_equilibrium_CX(**eqcxkw)  # The call to draw the equilibrium contours
+                if profile_axs is not None:
+                    plot_single_time_profile_set(
+                        ods=ods,
+                        t_idx=it,
+                        profile_axs=profile_axs,
+                        profile_x_quantity=profile_x_quantity,
+                        profile_y_quantities=profile_y_quantities,
+                        gentle=gentle,
+                        **eqkw,
+                    )
+        elif cx_ax is not None:
             printd('plot_single_cx_ods{}: this eq slice is INACTIVE; skipping eq plot', topic='efitviewer')
             it = 0
             cx_ax.set_aspect('equal')
@@ -1447,81 +1770,83 @@ def plot_single_cx_ods(
             cx_ax.xaxis.set_ticks_position('bottom')
             cx_ax.yaxis.set_ticks_position('left')
 
-        def overlay_customizations(ovls):
-            ovls = dict(ovls)
-            for ovl in ovls:
-                if ovls[ovl] and ovl in customization:
-                    # Replace "True" w/ custom overlay instructions (if present) but first do some processing
-                    custom = dict(customization[ovl])
-                    for item in list(custom.keys()):
-                        # Remove "None", since we were too busy to look up real defaults & they might not be None
-                        if custom[item] is None:
-                            if item == 't':
-                                custom[item] = t * 1e-3
-                            else:
-                                custom.pop(item, None)
-                        if item.startswith('pass_in_keywords_'):
-                            pikw = custom.pop(item, {})
-                            custom.update(pikw)
-                    # Special debugging
-                    if ovl in ['position_control', 'pulse_schedule']:
-                        custom['timing_ref'] = timing_ref
-                    # Custom instructions ready
-                    if len(custom):
-                        ovls[ovl] = custom
-                    else:
-                        ovls[ovl] = True
-            return ovls
+        if cx_ax is not None:
 
-        # Plot hardware overlay(s)
-        overlays = overlay_customizations(overlays)
-        scratch['overlays'] = overlays
-        old_font_size = copy.copy(rcParams['font.size'])
-        if default_fontsize is not None:
-            try:
-                rcParams['font.size'] = default_fontsize
-            except ValueError as exc:
-                printe(f"Could not set rcParams['font.size'] = {repr(default_fontsize)} " f"(default_fontsize), because of {exc}")
+            def overlay_customizations(ovls):
+                ovls = dict(ovls)
+                for ovl in ovls:
+                    if ovls[ovl] and ovl in customization:
+                        # Replace "True" w/ custom overlay instructions (if present) but first do some processing
+                        custom = dict(customization[ovl])
+                        for item in list(custom.keys()):
+                            # Remove "None", since we were too busy to look up real defaults & they might not be None
+                            if custom[item] is None:
+                                if item == 't':
+                                    custom[item] = t * 1e-3
+                                else:
+                                    custom.pop(item, None)
+                            if item.startswith('pass_in_keywords_'):
+                                pikw = custom.pop(item, {})
+                                custom.update(pikw)
+                        # Special debugging
+                        if ovl in ['position_control', 'pulse_schedule']:
+                            custom['timing_ref'] = timing_ref
+                        # Custom instructions ready
+                        if len(custom):
+                            ovls[ovl] = custom
+                        else:
+                            ovls[ovl] = True
+                return ovls
 
-        scratch['overlays'] = overlays
-        # Thomson scattering has a different default than others, so we must make sure blank is replaced by False
-        overlays.setdefault('thomson_scattering', False)
-        ods.plot_overlay(ax=cx_ax, **overlays)  # The call to do the overlays
-
-        # Plot special hardware overlay(s)
-        special_overlays = overlay_customizations(special_overlays)
-        scratch['special_overlays'] = special_overlays
-        for so, soa in special_overlays.items():
-            if soa:
-                skw = soa if isinstance(soa, dict) else {}
-                function_name = 'plot_{}_overlay'.format(so)
+            # Plot hardware overlay(s)
+            overlays = overlay_customizations(overlays)
+            scratch['overlays'] = overlays
+            old_font_size = copy.copy(rcParams['font.size'])
+            if default_fontsize is not None:
                 try:
-                    special_function = eval(function_name)
-                except NameError:
-                    printe('Unrecognized special overlay: {} has no function (expected {})'.format(so, function_name))
-                else:
-                    special_function(ax=cx_ax, ods=ods, **skw)
+                    rcParams['font.size'] = default_fontsize
+                except ValueError as exc:
+                    printe(f"Could not set rcParams['font.size'] = {repr(default_fontsize)} " f"(default_fontsize), because of {exc}")
 
-        # Annotations and finishing touches
-        actually_show_legend = (multi_cases and (show_legend > 0)) or (show_legend == 1)
-        actually_show_cornernote = (not multi_cases and (show_cornernote > 0)) or (show_cornernote == 1)
-        if actually_show_legend:
-            cx_ax.legend(loc=0)
-        if actually_show_cornernote:
-            cornernote(
-                ax=cx_ax,
-                device=device,
-                shot='{} {}'.format(shot, efitid),
-                time='{:0.2f}'.format(ods['equilibrium.time'][it] * 1000) if len(t) == 1 else ' ',
-            )
-        if xlim is not None:
-            cx_ax.set_xlim(xlim)
-        if ylim is not None:
-            cx_ax.set_ylim(ylim)
-        if len(t) > 1:
-            cx_ax.legend(loc=3)
+            scratch['overlays'] = overlays
+            # Thomson scattering has a different default than others, so we must make sure blank is replaced by False
+            overlays.setdefault('thomson_scattering', False)
+            ods.plot_overlay(ax=cx_ax, **overlays)  # The call to do the overlays
 
-        plot_style_enforcement(ax=cx_ax, plot_style=plot_style_kw)
+            # Plot special hardware overlay(s)
+            special_overlays = overlay_customizations(special_overlays)
+            scratch['special_overlays'] = special_overlays
+            for so, soa in special_overlays.items():
+                if soa:
+                    skw = soa if isinstance(soa, dict) else {}
+                    function_name = 'plot_{}_overlay'.format(so)
+                    try:
+                        special_function = eval(function_name)
+                    except NameError:
+                        printe('Unrecognized special overlay: {} has no function (expected {})'.format(so, function_name))
+                    else:
+                        special_function(ax=cx_ax, ods=ods, **skw)
+
+            # Annotations and finishing touches
+            actually_show_legend = (multi_cases and (show_legend > 0)) or (show_legend == 1)
+            actually_show_cornernote = (not multi_cases and (show_cornernote > 0)) or (show_cornernote == 1)
+            if actually_show_legend:
+                cx_ax.legend(loc=0)
+            if actually_show_cornernote:
+                cornernote(
+                    ax=cx_ax,
+                    device=device,
+                    shot='{} {}'.format(shot, efitid),
+                    time='{:0.2f}'.format(ods['equilibrium.time'][it] * 1000) if len(t) == 1 else ' ',
+                )
+            if xlim is not None:
+                cx_ax.set_xlim(xlim)
+            if ylim is not None:
+                cx_ax.set_ylim(ylim)
+            if len(t) > 1:
+                cx_ax.legend(loc=3)
+
+            plot_style_enforcement(ax=cx_ax, plot_style=plot_style_kw)
 
         if default_fontsize is not None:
             rcParams['font.size'] = old_font_size
@@ -1953,22 +2278,80 @@ def find_cq_options():
     return cq_opts, default_contour_quantity
 
 
-def find_profile_y_options(index=0):
+def find_profile_y_options(index=0, allow_2d=True, allow_constraints=True):
     """
     Finds a list of quantities that could go on the y axis of the profile plots
 
     :param index: int
         Index of time-slice to check in, in case different times have different availability (hopefully not)
 
+    :param allow_2d: bool
+        Allow 2D profiles to enter the list. These have to be sliced later to make 1D plots.
+
+    :param allow_constraints: bool
+        Allow constraint profiles to enter the list.
+        These have to have associated functions for interpreting constraint data properly.
+
     :return: tuple
         list: all options for y axis quantities
         list: list of recommended defaults for each plot panel
     """
     out_loc, settings_loc, out, settings, parent_settings, scratch = pick_output_location()
     yopts = [k for k, v in out['efitviewer_ods']['equilibrium']['time_slice'][index]['profiles_1d'].items() if isinstance(v, np.ndarray)]
+    if 'j_tor' not in yopts:
+        if index == 0:
+            ods = out['efitviewer_ods']
+        else:
+            ods = out[f'efitviewer_ods_{index}']
+        if calculate_j_toroidal(ods, 0, check_availability=True):
+            yopts += ['j_tor']
+    if allow_2d:
+        yopts += [
+            k for k, v in out['efitviewer_ods']['equilibrium']['time_slice'][index]['profiles_2d.0'].items() if isinstance(v, np.ndarray)
+        ]
+        if 'b_field_tor' not in yopts:
+            yopts += ['b_field_tor']
+    if allow_constraints:
+        for k in out['efitviewer_ods']['equilibrium']['time_slice'][index]['constraints']:
+            function_name = f'constraint_profile_{k}'
+            try:
+                eval(function_name)
+            except NameError:
+                pass
+            else:
+                yopts += [k]
     priorities = ['pressure', 'q', 'dpressure_dpsi', 'f_df_dpsi', 'phi']
     ydefault = [p for p in priorities if p in yopts]
     ydefault += [y for y in yopts if y not in ydefault]
     ydefault += yopts
     ydefault += [None] * 100
     return yopts, ydefault
+
+
+def setup_default_profiles(index=0):
+    """Loads a group of settings that will probably make for good-looking & interesting profiles"""
+    out_loc, settings_loc, out, settings, parent_settings, scratch = pick_output_location()
+
+    # Axis quantities
+    desired_quantities = ['pressure', 'q', 'dpressure_dpsi', 'f_df_dpsi']
+    quantities = [dq for dq in desired_quantities if dq in out['efitviewer_ods']['equilibrium']['time_slice'][index]['profiles_1d']]
+    p = settings.setdefault('profiles', SettingsName())
+    p['xaxis_quantity'] = 'rho_tor_norm'
+    for i, q in enumerate(quantities):
+        p[f'yaxis_quantity_{i}'] = q
+
+    # Subplot counts
+    settings['show_cx'] = True
+    p['num_profiles'] = len(quantities)
+    p['num_profile_cols'] = square_subplots(len(quantities), just_numbers=True)[1]
+
+    # Figure arrangement: size, spacing, etc.
+    settings['figsize'] = (10, 8)
+    p['gridspec_kw'] = {'width_ratios': [1 + p['num_profile_cols']] + [1] * p['num_profile_cols']}
+    layout_tag = f"cx-{p['num_profiles']}-{p['num_profile_cols']}"
+    layout = (
+        settings.setdefault('plot_style_kw', SettingsName())
+        .setdefault('subplot_layout', SettingsName())
+        .setdefault(layout_tag, SettingsName())
+    )
+    layout.update(dict(wspace=0.16, left=0.03, right=0.92))
