@@ -118,8 +118,11 @@ def omfit_file_formatter(filename, overwrite=True):
 
 if '__main__' == __name__:
     if len(sys.argv) == 1:
-        pass
+        sys.exit(0)
     elif len(sys.argv[1:]) == 1:
-        omfit_file_formatter(sys.argv[1])
+        out = omfit_file_formatter(sys.argv[1])
     else:
-        omfit_file_formatter(sys.argv[1:])
+        out = omfit_file_formatter(sys.argv[1:])
+    if out is None or (isinstance(out, dict) and all([o is None for o in out.values()])):
+        sys.exit(0)
+    sys.exit(1)
