@@ -107,9 +107,9 @@ class OMFITiniSection(SortedDict):
 class OMFITini(OMFITiniSection, OMFITascii):
     r"""
     OMFIT class used to interface with configuration files (INI files).
-    This types of files are used by codes such as BOUT++, IMFIT, IPS
+    This types of files are used by codes such as IMFIT and IPS
 
-    This class is based on the configobj class http://www.voidspace.org.uk/python/configobj.html
+    This class is based on the configobj class https://configobj.readthedocs.io/en/latest/index.html
 
     :param filename: filename passed to OMFITascii class
 
@@ -119,7 +119,8 @@ class OMFITini(OMFITiniSection, OMFITascii):
     def __init__(self, filename, **kw):
         OMFITascii.__init__(self, filename, **kw)
         SortedDict.__init__(self, caseInsensitive=True)
-        self.load()
+        self.list_values = True
+        self.dynaLoad = True
 
     @staticmethod
     def _interpret(section, key):
@@ -136,7 +137,7 @@ class OMFITini(OMFITiniSection, OMFITascii):
 
         :return: None
         """
-        tmp = configobj.ConfigObj(self.filename)
+        tmp = configobj.ConfigObj(self.filename, list_values=self.list_values)
         tmp.walk(self._interpret)
         self.clear()
         self._cast(tmp)
@@ -152,3 +153,11 @@ class OMFITini(OMFITiniSection, OMFITascii):
         tmp.filename = self.filename
         tmp.walk(self._encode)
         tmp.write()
+
+
+############################################
+if '__main__' == __name__:
+    test_classes_main_header()
+
+    tmp = OMFITini(f'{OMFITsrc}/../modules/EPED/eped.config')
+    tmp.load()
