@@ -101,6 +101,7 @@ def _read_rdb_password_from_file(server=None, filename=None):
         'd3dsqlsrvr': ['D3DRDB.sybase_login', 'd3drdb.sybase_login'],
         'd3sqlsrvr': ['D3SQLSRVR.sybase_login', 'd3sqlsrvr.sybase_login'],
         'east_database': ['east_database.sybase_login'],
+        'energy': ['ENERGY.sybase_login'],
         'nstxrdb': ['nstxlogs.sybase_login'],
     }.get(translate_RDBserver(server), [''])
     files = [filename] if filename is not None else files
@@ -138,7 +139,7 @@ def get_public_rdb_login(server):
     :param server: string
     :return: (string, string)
     """
-    default_user_pass = ('guest', 'guest_pwd')  # Applies to d3drdb, d3dsqlsrvr, d3sqlsrvr, east_database
+    default_user_pass = ('guest', 'guest_pwd')  # Applies to d3drdb, d3dsqlsrvr, d3sqlsrvr, east_database, energy
     return {'huez': ('d3dpub', 'd3dpub'), 'pppl_postgres': ('nstxpub', 'nstxpub')}.get(translate_RDBserver(server), default_user_pass)
 
 
